@@ -318,11 +318,14 @@ public:
 /*
 Provide rules such as:
 
-IMPLIES e1 (FORALL (SEQ decls) e2) 
--->
-FORALL (SEQ decls) (IMPLIES e1 e2) 
+1. IMPLIES e1 (FORALL (SEQ decls) e2) 
+   -->
+   FORALL (SEQ decls) (IMPLIES e1 e2) 
 
-provided decls don't capture any free variables in e1.
+   provided decls don't capture any free variables in e1.
+
+2. FORALL (SEQ decls1) (FORALL (SEQ decls2) e) 
+   --> FORALL (SEQ decls1 decls2) e
 
 */
 
@@ -369,30 +372,39 @@ set<string> getFreeVars (Node* n) {
 
 Node* liftQuants(FDLContext* c, Node* n) {
 
-    if (! (n->kind == IMPLIES && n->child(1)->kind == FORALL) ) return n;
+    if (n->kind == IMPLIES && n->child(1)->kind == FORALL) {
 
-    Node* e1 = n->child(0);
-    Node* decls = n->child(1)->child(0);
+        Node* e1 = n->child(0);
+        Node* decls = n->child(1)->child(0);
     
-    set<string> e1FreeVars = getFreeVars(e1);
+        set<string> e1FreeVars = getFreeVars(e1);
 
-    for (int i = 0; i != decls->arity(); i++) {
-        if ( e1FreeVars.count(decls->child(i)->id) != 0 ) {
+        for (int i = 0; i != decls->arity(); i++) {
+            if ( e1FreeVars.count(decls->child(i)->id) != 0 ) {
 
-            printMessage(WARNINGm,
-                         "FORALL lift of " + (decls->child(i)->id)
-                         + " inhibited to avoid capture");
-            return n;
+                printMessage(WARNINGm,
+                             "FORALL lift of " + (decls->child(i)->id)
+                             + " inhibited to avoid capture");
+                return n;
+            }
         }
+
+        n->kind = FORALL;
+        n->child(0) = decls;
+
+        n->child(1)->kind = IMPLIES;
+        n->child(1)->child(0) = e1;
+
     }
+    else if (n->kind == FORALL && n->child(1)->kind == FORALL) {
 
-    n->kind = FORALL;
-    n->child(0) = decls;
-
-    n->child(1)->kind = IMPLIES;
-    n->child(1)->child(0) = e1;
-
+        Node* decls1 = n->child(0);
+        Node* decls2 = n->child(1)->child(0);
+        Node* e = n->child(1)->child(1);
 
+        decls1->appendChildren(decls2);
+        n->child(1) = e;
+    }
     return n;
 }
 
@@ -433,19 +445,41 @@ void useTuplesForArrayIndexes(Node* unit) {
 }
 
 //========================================================================
-// Remove EQ and NE types
+// Remove EQ, NE and ITE type arguments
 //========================================================================
 
-Node* removeEqType(Node* n) {
-    if ((n->kind == EQ || n->kind == NE || n->kind == TERM_EQ)
-        && n->arity() == 3) {
+Node* removeTypeArg(Node* n) {
+
+    if ( ( (n->kind == EQ || n->kind == NE || n->kind == TERM_EQ)
+           && n->arity() == 3)
+         || (n->kind == ITE && n->arity() == 4)
+        )  {
         n->popChild();
     }
     return n;
 }
 
-void removeEqTypes(Node* unit) {
-    unit->mapOver1(removeEqType);
+void removeTypeArgs(Node* unit) {
+    unit->mapOver1(removeTypeArg);
+}
+
+//========================================================================
+// Remove Patterns from Quantifiers
+//========================================================================
+// Patterns are used in the axioms characterising operations on the bit type.
+// May be used elsewhere too.
+
+Node* removeQuantifierPattern(Node* n) {
+
+    if ( (n->kind == FORALL || n->kind == EXISTS)
+         && n->arity() == 3) {
+        n->popChild();
+    }
+    return n;
+}
+
+void removeQuantifierPatterns(Node* unit) {
+    unit->mapOver1(removeQuantifierPattern);
 }
 
 
@@ -622,9 +656,12 @@ translateUnit(FDLContext* ctxt, Node* unit) {
     // Eliminating enumeration type primitives
     //--------------------------------------------------------------------
 
-    if (option("abstract-enums")) abstractEnums(ctxt, unit);
-
-    if (option("elim-enums")) enumsToIntSubranges(ctxt, unit);
+    if (option("abstract-enums"))
+        abstractEnums(ctxt, unit);
+    else if (option("elim-enums"))
+        enumsToIntSubranges(ctxt, unit);
+    else if (option("axiomatise-enums"))
+        axiomatiseEnums(ctxt, unit);
 
     //--------------------------------------------------------------------
     // Early array and record abstraction
@@ -741,9 +778,13 @@ translateUnit(FDLContext* ctxt, Node* unit) {
     //--------------------------------------------------------------------
     // Minor final formatting in preparation for passing to drivers
     //--------------------------------------------------------------------
-    // - Remove type argument from EQ, NE, and TERM_EQ nodes.
+    // - Remove type argument from EQ, NE, TERM_EQ and ITE nodes.
 
-    removeEqTypes(unit);
+    removeTypeArgs(unit);
+
+    if (!option("include-patterns-in-quantifiers")) {
+        removeQuantifierPatterns(unit);
+    }
 
     //--------------------------------------------------------------------
     // Insert modified declarations back into unit AST.
@@ -760,11 +801,11 @@ translateUnit(FDLContext* ctxt, Node* unit) {
 //========================================================================
 
 string
-elaborateUnit(Node* unit) {
+elaborateUnit(Node* unit, UnitInfo* unitInfo) {
     printMessage(FINESTm, "Unit parse tree:" + ENDLs + unit->toString());
 
-    // Also modifies unit
-    FDLContext* ctxt = putUnitInStandardForm(unit);
+    // Modifies both unit and unitInfo
+    FDLContext* ctxt = putUnitInStandardForm(unit, unitInfo);
 
     if (ctxt == 0) return "FDL normalisation failed";
     
