@@ -759,7 +759,6 @@ Terms
         return oldN;
     }
     case ITE: {
-        oldN->popChild();  // Strip type arg.
         return oldN;
     }
 
@@ -799,6 +798,7 @@ Top level structure with no further changes
     case EXTRASORTS:
     case TCONST:
     case RULES:
+    case RULE:
     case GOALS:
     case GOAL:
     case HYPS:
@@ -1434,6 +1434,23 @@ SMTLibDriver::getResults(string& remarks) {
 
 }
 
+//---------------------------------------------------------------------------
+// finaliseGoal()
+//---------------------------------------------------------------------------
+
+void
+SMTLibDriver::finaliseGoal() {
+
+    if (option("delete-working-files")) {
+
+        tryRemoveFile(solverInputFileName);
+        tryRemoveFile(solverOutputFileName);
+        tryRemoveFile(solverErrorFileName);
+    }
+    return;
+
+    
+}
 
 
 //=========================================================================
