@@ -27,8 +27,8 @@ LICENSE.txt and online at http://www.gnu.org/licenses/.
 
 #include "utility.hh"
 
-
-#include <iostream>
+// Moved to utility.hh
+// #include <iostream>
 using std::endl;
 using std::cout;
 using std::cerr;
@@ -37,7 +37,8 @@ using std::cerr;
 using std::ostringstream;
 using std::istringstream;
 
-#include <fstream>
+// Moved to utility.hh
+// #include <fstream>
 using std::ofstream;
 
 #include <map>
@@ -61,18 +62,31 @@ using std::isdigit;
 
 #include <stdexcept>  // for runtime_error
 
-extern "C" { // For access, mkdir, gethostname, getpid functions
+extern "C" {
 
-#include <sys/stat.h>
-#include <unistd.h>
+#include <sys/stat.h> // For mkdir
+#include <unistd.h> // For access, unlink, gethostname, getpid
+#include <dirent.h> // For opendir, readdir, closedir
+#include <errno.h>  // For errno static var and error codes
+#include <string.h> // For strerror - printing error codes
 
 #ifdef _WIN32
+
 #include <windows.h>
+
+#else
+
+#include <sys/resource.h> // For getrusage() resource name argument
+#include <sys/times.h>    // For getrusage()
+
 #endif
 }
 
+
+
+
 //========================================================================
-// String functions and constants 
+// String functions and constants
 //========================================================================
 
 // Utility function for debugging. Normal C++ STL always inserts
@@ -147,7 +161,7 @@ boolToString(bool b) {
 const string ENDLs ("\n");
 
 
-bool 
+bool
 hasSuffix(const string& s, const string& t) {
 
     string::size_type i = s.rfind(t, s.size());
@@ -158,7 +172,7 @@ hasSuffix(const string& s, const string& t) {
     // cast to -1.
 }
 
-bool 
+bool
 hasPrefix(const string& s, const string& t) {
 
     string::size_type i = s.find(t, 0);
@@ -167,12 +181,12 @@ hasPrefix(const string& s, const string& t) {
         && i == 0;
 }
 
-bool 
+bool
 hasSubstring(const string& s, const string& t) {
 
     string::size_type i = s.find(t, 0);
     return i != string::npos;
-}    
+}
 
 
 
@@ -184,6 +198,17 @@ appendCommaString(string& s, const string& t) {
     return;
 }
 
+
+// if q == "/..." or p == "" then q else p + "/" + q
+string joinPaths(const string& p, const string& q) {
+
+    if ((q.size() > 0 && q[0] == '/') || p.size() == 0)
+        return q;
+    else
+        return p + "/" + q;
+}
+
+
 bool
 hasUpperCaseStart(const string& s) {
     return s.size() >= 0 && 'A' <= s[0] && s[0] <= 'Z';
@@ -196,10 +221,10 @@ member(char c, const string& s) {
     return cpos != string::npos;
 }
 
-// returns substring before first occurrence of c. 
-// If no occurrence, returns whole string.    
+// returns substring before first occurrence of c.
+// If no occurrence, returns whole string.
 
-string 
+string
 substringBefore(char c, const string& s) {
     string result(s);
     string::size_type cpos = s.find(c);
@@ -213,9 +238,9 @@ substringBefore(char c, const string& s) {
 
 
 
-// returns substring after first occurrence of c. 
-// If no occurrence, returns empty string.    
-string 
+// returns substring after first occurrence of c.
+// If no occurrence, returns empty string.
+string
 substringAfter(char c, const string& s) {
     string result(s);
     string::size_type cpos = s.find(c);
@@ -350,18 +375,21 @@ string fixSuffix(const string& s,
 map<string, vector<string> > commandLineOptions;
 
 vector<string> commandLineOptionsList;
+vector<string> plainCommandLineOptionsList; // Same as above, but for
+					    // showing the supplied
+					    // options in -plain mode.
 vector<string> nonOptionArgs;
 
-/* 
+/*
 processCommandArgs initialises the commandLineOptions map:
 
 Option             Effect
 
    -<key>            "" added to end of list of map entries for <key>
 
-   -<key>=        
- | -<key>=none    
- | -<key>=default 
+   -<key>=
+ | -<key>=none
+ | -<key>=default
  | -<key>=empty      Option map entry for <key> is erased
 
    -<key>=<val>      <val> added to end of list of map entries for <key>
@@ -371,7 +399,7 @@ least one entry.
 
 */
 
-vector<string> 
+vector<string>
 processCommandArgs (int argc, char *argv[]) {
 
 
@@ -411,6 +439,15 @@ processCommandArgs (int argc, char *argv[]) {
                  make_pair(key, vector<string>(1,val))
                  );
 
+	    // Check if the value is looking like a path; if so don't
+	    // show it in -plain mode.
+	    bool is_path = (val.find("/") != string::npos) || (val.find("\\") != string::npos);
+	    if (is_path) {
+	      plainCommandLineOptionsList.push_back(key + "=[SUPRESSED IN PLAIN MODE]");
+	    } else {
+	      plainCommandLineOptionsList.push_back(arg);
+	    }
+
             if (! insResult.second) {
                 // If already one or more entries for key in map,
                 // insertion did not take place.
@@ -420,7 +457,7 @@ processCommandArgs (int argc, char *argv[]) {
                 oldMapEntry->second.push_back(val);
             }
 
-            
+
         } else {
             nonOptionArgs.push_back(arg);
         }
@@ -431,14 +468,14 @@ processCommandArgs (int argc, char *argv[]) {
 // Return true just when option set and value is not
 // "false", "none" or "default".
 
-bool 
+bool
 option(const string& s) {
     map<string, vector<string> >::iterator i = commandLineOptions.find(s);
 
     return i != commandLineOptions.end();
 }
 
-string 
+string
 optionVal(const string& s) {
     map<string, vector<string> >::iterator i = commandLineOptions.find(s);
     if (i != commandLineOptions.end()) {
@@ -556,7 +593,7 @@ string withTimeoutAndIO(const string &cmd,
 // <gstr>             <g>,0              Select goal <g> all concls.
 
 // <unit-info> ::= <unit-name> <unit-option>*
-// <unit-option> ::= 
+// <unit-option> ::=
 //  <tag>?<val>
 // | <val>
 //
@@ -572,16 +609,31 @@ string withTimeoutAndIO(const string &cmd,
 
 UnitInfo::Status UnitInfo::status = UnitInfo::BEFORE_RANGE;
 
-UnitInfo::UnitInfo(string s) {
-    vector<string> toks(tokeniseString(s));
+UnitInfo::UnitInfo(const string& unitInfoStr)
+    : dirRLURulesEnd(0), unitRLURulesEnd(0) {
+
+    vector<string> toks(tokeniseString(unitInfoStr));
 
     unitName = toks[0];
 
-    { 
+    {
+
+        if (option("prefix")) unitPathPrefix = optionVal("prefix");
+        // else unitPathPrefix == "" by default
+
         vector<string> unitNameParts = splitString(unitName,"/");
         unitFileName = unitNameParts.back();
         unitNameParts.pop_back();
-        unitPathName = concatStrings(unitNameParts, "/");
+        unitPath = concatStrings(unitNameParts, "/");
+        if (unitNameParts.size() > 0) {
+            unitDirName = unitNameParts.back();
+        }
+        else if (unitPathPrefix.size() > 0) {
+            vector<string> prefixParts = splitString(unitPathPrefix, "/");
+            unitDirName = prefixParts.back();
+        }
+        // else unitDirName == "" by default
+
     }
 
     for (int i = 1; i != (int) toks.size(); i++ ) {
@@ -597,7 +649,7 @@ UnitInfo::UnitInfo(string s) {
             // Otherwise skip option
 
             string tag (splitToken.at(0));
-            
+
             vector<string> activeTags;
             if (option("active-unit-tags")) {
                 activeTags = splitString(optionVal("active-unit-tags"), ":");
@@ -643,7 +695,7 @@ UnitInfo::UnitInfo(string s) {
     bool atToLimit = toUnitGiven ? optionVal("to-unit") == unitName
                                  : false;
 
-    
+
     // Logic here is to have unitIncluded true in range between from-unit
     // and to-unit inclusive, with range extended to end when respective
     // limit is missing.
@@ -681,11 +733,11 @@ UnitInfo::UnitInfo(string s) {
 bool
 UnitInfo::include(int goal, int concl) {
 
-    bool goalSliceSelected = 
+    bool goalSliceSelected =
         selectedSet.find(make_pair(goal, concl)) != selectedSet.end()
         || selectedSet.find(make_pair(goal, 0)) != selectedSet.end();
 
-    
+
     if (option("include-selected-goals")) {
         return
             unitIncluded
@@ -718,8 +770,8 @@ UnitInfo::includeUnit() {
 string currentUnit;         // [<path>/]<fileroot>
 string currentUnitPath;     //  <path>
 string currentUnitFile;     //  <fileroot>
-string currentUnitKind;     //  procedure | function | task_type 
-string currentGoalNum;      //  <goal number> 
+string currentUnitKind;     //  procedure | function | task_type
+string currentGoalNumStr;      //  <goal number>
 string currentGoalOrigins;  //  Info about where in program goal comes from
 
 
@@ -728,41 +780,41 @@ updateGoalInfo expects string of form:
 
 [function_|procedure_|task_type_]<name>_<num> <desc>
 
-where 
+where
 
 <desc> ::= path(s) from <a> to <b>
          | checks of refinement integrity
-<a> ::= 
+<a> ::=
 
-   start                                          
- | from assertion of line 49                      
- | from default assertion of line 49              
+   start
+ | from assertion of line 49
+ | from default assertion of line 49
 
-<b> ::= 
+<b> ::=
    finish
- | run-time check associated with [\n][ ]+ statement of line 201   
- | assertion of line 49 
- | default assertion of line 49 
- | precondition check associated with [\n][ ]+ statement of line xx:  
+ | run-time check associated with [\n][ ]+ statement of line 201
+ | assertion of line 49
+ | default assertion of line 49
+ | precondition check associated with [\n][ ]+ statement of line xx:
 ;
 Current version just captures minimum.  Could also do abbreviations and
 line numbers.
 
-POGS .sum files use 
+POGS .sum files use
 
-From::= 
+From::=
   start
- | <nn>   
+ | <nn>
  | <>           (empty) (in case of refinement)
 ;
 
-To::= 
+To::=
 
    rtc check @ <nn>
  | assert @ finish
  | assert @ <nn>
  | refinement
- | pre check @ 1514 
+ | pre check @ 1514
 ;
 POGS doesn't seem to distinguish between assertions and default assertions.
 
@@ -771,13 +823,13 @@ POGS doesn't seem to distinguish between assertions and default assertions.
 
 // Called at start of processUnit in main.cc
 
-void initCurrentUnitInfo(UnitInfo unitInfo) {  
+void initCurrentUnitInfo(UnitInfo* unitInfo) {
 
-    currentUnit = unitInfo.getUnitName();
-    currentUnitPath = unitInfo.getUnitPathName();
-    currentUnitFile = unitInfo.getUnitFileName();
+    currentUnit = unitInfo->getUnitName();
+    currentUnitPath = unitInfo->getUnitPath();
+    currentUnitFile = unitInfo->getUnitFileName();
 
-    currentGoalNum = "";
+    currentGoalNumStr = "";
     currentGoalOrigins = ",";  // "," since two fields in report
     currentUnitKind = "";
 
@@ -787,22 +839,30 @@ void initCurrentUnitInfo(UnitInfo unitInfo) {
 }
 
 void updateCurrentGoalInfo(const string& s) {
+    extractGoalInfo(s, currentUnitKind, currentGoalNumStr, currentGoalOrigins);
+    return;
+}
+
+void extractGoalInfo(const string& s,
+                     string& unitKind,
+                     string& goalNum,
+                     string& goalOrigins) {
 
      vector<string> ss = tokeniseString(s);
-     string currentGoalName = ss[0];     //  <kind>_<fileroot>_<goal number> 
+     string currentGoalName = ss[0];     //  <kind>_<fileroot>_<goal number>
 
      vector<string> goalNameParts = splitString(currentGoalName, "_");
 
      // Could grab unit kind instead from VCG/SIV file header. Cleaner
 
-     currentUnitKind = goalNameParts.front();
-     if (currentUnitKind == "task") currentUnitKind = "task_type";
+     unitKind = goalNameParts.front();
+     if (unitKind == "task") unitKind = "task_type";
 
-     currentGoalNum = goalNameParts.back();
+     goalNum = goalNameParts.back();
 
      string from;
      string to;
- 
+
      if (ss[1] == "path(s)") {
          vector<string>::iterator i = std::find(ss.begin(), ss.end(), "to");
 
@@ -825,10 +885,10 @@ void updateCurrentGoalInfo(const string& s) {
          to = "*** Unrecognised goal description ***";
      }
 
-     currentGoalOrigins = from + "," + to;
+     goalOrigins = from + "," + to;
      return;
 }
-                     
+
 //========================================================================
 // Report file management
 //========================================================================
@@ -876,7 +936,7 @@ void openReportFiles() {
 	std::strftime(the_time, sizeof (the_time),
 		      "%d-%b-%Y, %H:%M:%S",
 		      timeinfo);
-	
+
 	logStream << "Date: " << the_time << endl;
     } else {
         logStream << endl;
@@ -889,7 +949,9 @@ void openReportFiles() {
 #else
     gethostname(hostname, 100);
 #endif
-    logStream << "Host: " << hostname << endl;
+    if (! option("plain")) {
+        logStream << "Host: " << hostname << endl;
+    }
 
     logStream << "Non-option args: " << endl;
     for (int i = 0; i != (int) nonOptionArgs.size(); i++) {
@@ -898,12 +960,20 @@ void openReportFiles() {
 
     logStream << "Option args: " << endl;
 
-    for (int i = 0; i != (int) commandLineOptionsList.size(); i++) {
-
+    if (option("plain")) {
+      for (int i = 0; i != (int) plainCommandLineOptionsList.size(); i++) {
         logStream << "    -"
-                 << commandLineOptionsList.at(i)
-                 << "   \\"
-                 << endl;
+		  << plainCommandLineOptionsList.at(i)
+		  << "   \\"
+		  << endl;
+      }
+    } else {
+      for (int i = 0; i != (int) commandLineOptionsList.size(); i++) {
+        logStream << "    -"
+		  << commandLineOptionsList.at(i)
+		  << "   \\"
+		  << endl;
+      }
     }
     logStream << endl;
 
@@ -931,7 +1001,7 @@ const int FINEm = 3;
 const int FINERm = 2;
 const int FINESTm = 1;
 
-int messageThreshold; 
+int messageThreshold;
 
 
 int currentConcl;
@@ -958,9 +1028,9 @@ void printMessageWithHeader(const string& header, const string& message) {
     s += "unit: " + currentUnit;
 
 
-    if (currentGoalNum.size() > 0) {
+    if (currentGoalNumStr.size() > 0) {
 
-        s += "  goal: " + currentGoalNum;
+        s += "  goal: " + currentGoalNumStr;
     }
     if (currentConcl > 0  && !option("fuse-concls") ) {
         s += "  concl: " + intToString(currentConcl);
@@ -993,7 +1063,7 @@ int printMessageAux1(int messageLevel, const string& message) {
     else if (messageLevel == FINERm)   header = "FINER";
     else if (messageLevel == FINEm)    header = "FINE";
     else if (messageLevel == INFOm)    header = "INFO";
-    else if (messageLevel == WARNINGm) header = "WARNING"; 
+    else if (messageLevel == WARNINGm) header = "WARNING";
     else                               header = "ERROR";
 
     printMessageAux2(messageLevel);
@@ -1024,9 +1094,9 @@ int printMessageOnOptionAux(const string& opt, const string& message) {
 
 
 //========================================================================
-// CSV formatting 
+// CSV formatting
 //========================================================================
-// For CSV format, see http://en.wikipedia.org/wiki/Comma-separated_values 
+// For CSV format, see http://en.wikipedia.org/wiki/Comma-separated_values
 
 //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 // csvDigest
@@ -1034,7 +1104,7 @@ int printMessageOnOptionAux(const string& opt, const string& message) {
 // Break up CSV string into its components
 
 // Basic. Should also:
-// - handle quoted " chars 
+// - handle quoted " chars
 // - strip head and tail whitespace from each value.
 
 vector<string> csvDigest(const string& s) {
@@ -1112,19 +1182,37 @@ string csvConcat(const vector<string>& ss) {
 
 void
 printCSVRecord(const string& status, const string& remarks) {
+    printCSVRecordAux(currentUnitKind,
+                      currentGoalOrigins,
+                      currentGoalNumStr,
+                      currentConcl,
+                      status,
+                      goalSliceTime,
+                      remarks);
+    return;
+}
+
+void
+printCSVRecordAux(const string& unitKind,
+                  const string& goalOrigins,
+                  const string& goalNumString,
+                  int conclNum,
+                  const string& status,
+                  const string& queryTime,
+                  const string& remarks) {
 
     if (! option("gstime")) goalSliceTime = "";
 
     string kindString(option("csv-reports-include-unit-kind")
-                      ? currentUnitKind
+                      ? unitKind
                       : "");
 
     string originsString(option("csv-reports-include-goal-origins")
-                       ? currentGoalOrigins
+                       ? goalOrigins
                        : ",");
 
-    string conclString(currentConcl > 0 && !option("fuse-concls")
-                       ? intToString(currentConcl)
+    string conclString(conclNum > 0 && !option("fuse-concls")
+                       ? intToString(conclNum)
                        : "");
 
     string quotedRemarks(remarks.size() > 0
@@ -1136,10 +1224,10 @@ printCSVRecord(const string& status, const string& remarks) {
               << currentUnitFile << ","  //  2
               << kindString << ","       //  3
               << originsString << ","    //  4, 5
-              << currentGoalNum << ","   //  6
+              << goalNumString << ","    //  6
               << conclString << ","      //  7
               << status << ","           //  8
-              << goalSliceTime  << ","   //  9
+              << queryTime  << ","       //  9
               << quotedRemarks << ","    // 10
               << currentHypsKinds << "," // 11
               << currentConclKinds       // 12
@@ -1213,12 +1301,12 @@ printStats() {
     }
 
     logStream << outStream.str();
- 
+
     if (option("echo-final-stats")) {
 
         cout << outStream.str() << endl;
 
-        // Add on root of report files. 
+        // Add on root of report files.
 
         string reportName("report");
         if (option("report")) reportName = optionVal("report");
@@ -1228,7 +1316,7 @@ printStats() {
         }
         cout << "Report files: " << reportName << ".*" << endl;
     }
-   
+
     sumStream << reportName << ",";
     sumStream << numErrorMessages << "," << numWarningMessages << ",";
     sumStream << setprecision(1);
@@ -1246,7 +1334,9 @@ printStats() {
     sumStream << setw(4) << fTimeoutConcls << "," ;
     sumStream << setw(4) << fErrorConcls << "," ;
 
-    sumStream << totalTime.toString();
+    if (!option("plain")) {
+      sumStream << totalTime.toString();
+    }
     sumStream << endl;
 
 
@@ -1257,20 +1347,62 @@ printStats() {
 //========================================================================
 // Timing
 //========================================================================
-// A previous implementation used val returned by clock() and 
-// divided by CLOCKS_PER_SEC.  with CLOCKS_PER_SEC = 1000000,
-// this had the disadvantage that time would wrap in under an hour.
-//
-// clock() also excludes time spent in child processes, which is needed
-// in some cases.
+/*
+Previous versions
+-----------------
+1. Using clock (3).
+This returned a clock_t value (32 bit unsigned on 32-bit system, 64
+bit unsigned on 64-bit system it seems from experiments.
 
-// clock_t val returned by times() calls is an elapsed real time and
-// not of much use.
+For time in sec had to divide by CLOCKS_PER_SEC = 1000000, with
+disadvantage that time would wrap in under an hour on 32 bit
+systems.
 
-// Get the OS times (user, kernel) for the current process.
+clock() also doesn't give breakdown of time into user and system
+time, and doesn't give time spent in child processes.
 
-static void
-GetOSTimes (struct tms *tOS)
+2. Using times (2).
+
+Returns a
+           struct tms {
+               clock_t tms_utime;  // user time
+               clock_t tms_stime;  // system time
+               clock_t tms_cutime; // user time of children
+               clock_t tms_cstime; // system time of children
+           };
+
+where must divide by sysconf(_SC_CLK_TCK) to get time in seconds.
+With SL5 and SL6, this is 100, giving a resolution of 0.01sec.
+This is rather coarse, given some SMT solver run times are of this
+order.
+
+Current version
+---------------
+Uses getrusage (2).
+
+time (7) remarks this has a clock resolution of HZ which can be
+0.01, 0.001 sec a couple of values inbetween, depending on kernel
+configuration.  The SL6 appears to use 0.001, so better than
+times().
+
+A local class Time is defined, similar to struct tms, except that it
+definitely has 64 bit values for the fields.
+
+struct rusage {
+     struct timeval ru_utime; // user time used
+     struct timeval ru_stime; // system time used
+     ...
+}
+struct timeval
+  {
+    __time_t tv_sec;		// Seconds.
+    __suseconds_t tv_usec;	// Microseconds.
+  };
+
+*/
+
+void
+Timer::getOSTimes (Timer::Time *tOS)
 {
 #ifdef _WIN32
   FILETIME CreationTime;
@@ -1285,65 +1417,96 @@ GetOSTimes (struct tms *tOS)
 
   conv.u.LowPart = UserTime.dwLowDateTime;
   conv.u.HighPart = UserTime.dwHighDateTime;
-  tOS->tms_utime = (long long) conv.QuadPart;
+  tOS->uTime = (long long) conv.QuadPart;
 
   conv.u.LowPart = KernelTime.dwLowDateTime;
   conv.u.HighPart = KernelTime.dwHighDateTime;
-  tOS->tms_stime = (long long) conv.QuadPart;
+  tOS->sTime = (long long) conv.QuadPart;
   // There is no notion of child process on windows, set this to 0
-  tOS->tms_cutime = 0LL;
-  tOS->tms_cstime = 0LL;
+  tOS->cuTime = 0ULL;
+  tOS->csTime = 0ULL;
 #else
-  times(tOS);
+// To use times() again.
+/*
+  struct tms tms_time;
+  times(&tms_time);
+  tOS->sTime = tms_time->tms_stime;
+  tOS->uTime = tms_time->tms_utime;
+  tOS->csTime = tms_time->tms_cstime;
+  tOS->cuTime = tms_time->tms_cutime;
+*/
+  struct rusage selfRUsage;
+  struct rusage childRUsage;
+
+  getrusage(RUSAGE_SELF, &selfRUsage);
+  getrusage(RUSAGE_CHILDREN, &childRUsage);
+
+  timeval sT = selfRUsage.ru_stime;
+  timeval uT = selfRUsage.ru_utime;
+  timeval csT = childRUsage.ru_stime;
+  timeval cuT = childRUsage.ru_utime;
+
+  tOS->sTime = ((unsigned long long int) sT.tv_sec) * 1000000 + sT.tv_usec;
+  tOS->uTime = ((unsigned long long int) uT.tv_sec) * 1000000 + uT.tv_usec;
+  tOS->csTime = ((unsigned long long int) csT.tv_sec) * 1000000 + csT.tv_usec;
+  tOS->cuTime = ((unsigned long long int) cuT.tv_sec) * 1000000 + cuT.tv_usec;
+
 #endif
 }
 
-Timer::Timer() { GetOSTimes(&startTimeTuple); }
+Timer::Timer() {
+#ifdef _WIN32
+    // On Windows the granularity of a FILETIME is 100-nanosecond.
+    ticksPerSec = 10000000;
+#else
+    // If using times()
+    // ticksPerSec = sysconf(_SC_CLK_TCK);
+    // If using getrusage()
+    ticksPerSec = 1000000;     // This is the resolution of Time values,
+                               // *not* the timer resolution
 
+#endif
+    getOSTimes(&startTimeTuple);
+}
 void
-Timer::restart() { GetOSTimes(&startTimeTuple); }
+Timer::restart() { getOSTimes(&startTimeTuple); }
 
 void
 Timer::grabTimes() {
-    struct tms endTimeTuple;
-#ifdef _WIN32
-    // On Windows the granularity of a FILETIME is 100-nanosecond.
-    int ticks_per_sec = 10000000;
-#else
-    int ticks_per_sec = sysconf(_SC_CLK_TCK);
-#endif
-    GetOSTimes(&endTimeTuple);
-    
+    Time endTimeTuple;
+    getOSTimes(&endTimeTuple);
+
     uTime =
-        ((double) (endTimeTuple.tms_utime - startTimeTuple.tms_utime))
-        / ticks_per_sec;
+        ((double) (endTimeTuple.uTime - startTimeTuple.uTime))
+        / ticksPerSec;
 
     sTime =
-        ((double) (endTimeTuple.tms_stime - startTimeTuple.tms_stime))
-        / ticks_per_sec;
+        ((double) (endTimeTuple.sTime - startTimeTuple.sTime))
+        / ticksPerSec;
 
     cuTime =
-        ((double) (endTimeTuple.tms_cutime - startTimeTuple.tms_cutime))
-        / ticks_per_sec;
+        ((double) (endTimeTuple.cuTime - startTimeTuple.cuTime))
+        / ticksPerSec;
 
     csTime =
-        ((double) (endTimeTuple.tms_cstime - startTimeTuple.tms_cstime))
-        / ticks_per_sec;
+        ((double) (endTimeTuple.csTime - startTimeTuple.csTime))
+        / ticksPerSec;
 
 }
 
-string 
+string
 Timer::toString() {
 
     grabTimes();
     ostringstream oss;
+    oss << setprecision(3) << fixed;
     oss << (uTime + sTime + cuTime + csTime);
     return oss.str();
 }
 
-string 
+string
 Timer::toLongString() {
-    
+
     grabTimes();
 
     double totalTime = uTime + sTime + cuTime + csTime;
@@ -1352,7 +1515,7 @@ Timer::toLongString() {
 
     double percentChild = childTime * 100 / totalTime;
     double percentSys = sysTime * 100 / totalTime;
-    
+
     ostringstream oss;
     oss << setprecision(2)
         << fixed
@@ -1366,15 +1529,67 @@ Timer::toLongString() {
         << percentChild << "% ch, "
         << percentSys   << "% sys)"
         << endl;
-    
+
     return oss.str();
 }
 
 
 //==========================================================================
-// Manipulating names for working files
+// File system access & manipulating names for working files
 //==========================================================================
 
+vector<string> listDir (const string& dir) {
+    DIR *dp;
+    struct dirent *ep;
+    vector<string> contents;
+
+    dp = opendir (dir.c_str());
+
+    if (dp == NULL) {
+        cerr << "Error on trying to list directory " << dir << endl;
+        return contents;
+    }
+
+    while ( (ep = readdir (dp)) ) {
+        contents.push_back(ep->d_name);
+    }
+    (void) closedir (dp);
+    return contents;
+}
+
+// Returns  0 if remove successful.
+// Returns -1 on failure and sets errNumber and errorMessage.
+
+int removeFile (const string& file, int& errNumber, string& errorMessage) {
+
+    int failure = unlink(file.c_str());
+    if (failure != 0) {
+        errorMessage = string( strerror(errno));
+    }
+    return failure;
+}
+
+// Attempt to remove file.  Silent if file doesn't exist.
+
+void tryRemoveFile (const string& file) {
+
+    int errNumber;
+    string errMessage;
+    int failure = removeFile(file, errNumber, errMessage);
+    if (failure != 0 && errNumber != ENOENT) {
+
+        printMessage(ERRORm, "Attempt at removing file " + file +
+                     " failed with error: " + errMessage);
+    }
+    return;
+}
+
+
+
+
+bool readableFileExists(const string& s) {
+    return (access(s.c_str(), R_OK) == 0);
+}
 
 void ensureDirPathExists(const string& s) {
 
@@ -1416,9 +1631,9 @@ string flattenPathName(const string& s) {
 }
 
 string
-getFullGoalFileRoot(const string& defaultFileRoot, 
+getFullGoalFileRoot(const string& defaultFileRoot,
                     const string& unitName, // = dir1/.../dirn/unitFileRoot
-                    int goalNum, 
+                    int goalNum,
                     int conclNum) {
 
 
@@ -1447,8 +1662,8 @@ getFullGoalFileRoot(const string& defaultFileRoot,
 #else
         gethostname(hostname, 100);
 #endif
-        goalName = defaultFileRoot + "-" 
-                   + hostname + "-" 
+        goalName = defaultFileRoot + "-"
+                   + hostname + "-"
                    + intToString(getpid());
     }
     else {
@@ -1457,7 +1672,7 @@ getFullGoalFileRoot(const string& defaultFileRoot,
 
     // Join working-dir to goalName
 
-    string fullGoalName = goalRootDir + "/" + goalName;  
+    string fullGoalName = goalRootDir + "/" + goalName;
 
     // Ensure existence of directories mentioned in fullGoalName
 
